@@ -5,6 +5,9 @@ from .base_manager import BaseManager
 
 class AsyncManager(BaseManager):
     """Manage a client list for an asyncio server."""
+    async def can_disconnect(self, sid, namespace):
+        return self.is_connected(sid, namespace)
+
     async def emit(self, event, data, namespace, room=None, skip_sid=None,
                    callback=None, **kwargs):
         """Emit a message to a single client, a room, or all the clients
@@ -17,13 +20,13 @@ class AsyncManager(BaseManager):
         tasks = []
         if not isinstance(skip_sid, list):
             skip_sid = [skip_sid]
-        for sid in self.get_participants(namespace, room):
+        for sid, eio_sid in self.get_participants(namespace, room):
             if sid not in skip_sid:
                 if callback is not None:
-                    id = self._generate_ack_id(sid, namespace, callback)
+                    id = self._generate_ack_id(sid, callback)
                 else:
                     id = None
-                tasks.append(self.server._emit_internal(sid, event, data,
+                tasks.append(self.server._emit_internal(eio_sid, event, data,
                                                         namespace, id))
         if tasks == []:  # pragma: no cover
             return
@@ -36,19 +39,19 @@ class AsyncManager(BaseManager):
         """
         return super().close_room(room, namespace)
 
-    async def trigger_callback(self, sid, namespace, id, data):
+    async def trigger_callback(self, sid, id, data):
         """Invoke an application callback.
 
         Note: this method is a coroutine.
         """
         callback = None
         try:
-            callback = self.callbacks[sid][namespace][id]
+            callback = self.callbacks[sid][id]
         except KeyError:
             # if we get an unknown callback we just ignore it
             self._get_logger().warning('Unknown callback received, ignoring.')
         else:
-            del self.callbacks[sid][namespace][id]
+            del self.callbacks[sid][id]
         if callback is not None:
             ret = callback(*data)
             if asyncio.iscoroutine(ret):
