@@ -42,7 +42,7 @@ class Client(object):
                          reconnect to the server after an interruption, or
                          ``False`` to not reconnect. The default is ``True``.
     :param reconnection_attempts: How many reconnection attempts to issue
-                                  before giving up, or 0 for infinity attempts.
+                                  before giving up, or 0 for infinite attempts.
                                   The default is 0.
     :param reconnection_delay: How long to wait in seconds before the first
                                reconnection attempt. Each successive attempt
@@ -120,6 +120,7 @@ class Client(object):
 
         self.connection_url = None
         self.connection_headers = None
+        self.connection_auth = None
         self.connection_transports = None
         self.connection_namespaces = []
         self.socketio_path = None
@@ -131,8 +132,9 @@ class Client(object):
         self.namespace_handlers = {}
         self.callbacks = {}
         self._binary_packet = None
+        self._connect_event = None
         self._reconnect_task = None
-        self._reconnect_abort = self.eio.create_event()
+        self._reconnect_abort = None
 
     def is_asyncio_based(self):
         return False
@@ -232,14 +234,25 @@ class Client(object):
         self.namespace_handlers[namespace_handler.namespace] = \
             namespace_handler
 
-    def connect(self, url, headers={}, transports=None,
-                namespaces=None, socketio_path='socket.io'):
+    def connect(self, url, headers={}, auth=None, transports=None,
+                namespaces=None, socketio_path='socket.io', wait=True,
+                wait_timeout=1):
         """Connect to a Socket.IO server.
 
         :param url: The URL of the Socket.IO server. It can include custom
-                    query string parameters if required by the server.
+                    query string parameters if required by the server. If a
+                    function is provided, the client will invoke it to obtain
+                    the URL each time a connection or reconnection is
+                    attempted.
         :param headers: A dictionary with custom headers to send with the
-                        connection request.
+                        connection request. If a function is provided, the
+                        client will invoke it to obtain the headers dictionary
+                        each time a connection or reconnection is attempted.
+        :param auth: Authentication data passed to the server with the
+                     connection request, normally a dictionary with one or
+                     more string key/value pairs. If a function is provided,
+                     the client will invoke it to obtain the authentication
+                     data each time a connection or reconnection is attempted.
         :param transports: The list of allowed transports. Valid transports
                            are ``'polling'`` and ``'websocket'``. If not
                            given, the polling transport is connected first,
@@ -250,18 +263,27 @@ class Client(object):
         :param socketio_path: The endpoint where the Socket.IO server is
                               installed. The default value is appropriate for
                               most cases.
-
-        Note: The connection mechannism occurs in the background and will
-        complete at some point after this function returns. The connection
-        will be established when the ``connect`` event is invoked.
+        :param wait: if set to ``True`` (the default) the call only returns
+                     when all the namespaces are connected. If set to
+                     ``False``, the call returns as soon as the Engine.IO
+                     transport is connected, and the namespaces will connect
+                     in the background.
+        :param wait_timeout: How long the client should wait for the
+                             connection. The default is 1 second. This
+                             argument is only considered when ``wait`` is set
+                             to ``True``.
 
         Example usage::
 
             sio = socketio.Client()
             sio.connect('http://localhost:5000')
         """
+        if self.connected:
+            raise exceptions.ConnectionError('Already connected')
+
         self.connection_url = url
         self.connection_headers = headers
+        self.connection_auth = auth
         self.connection_transports = transports
         self.connection_namespaces = namespaces
         self.socketio_path = socketio_path
@@ -274,14 +296,33 @@ class Client(object):
         elif isinstance(namespaces, str):
             namespaces = [namespaces]
         self.connection_namespaces = namespaces
+        self.namespaces = {}
+        if self._connect_event is None:
+            self._connect_event = self.eio.create_event()
+        else:
+            self._connect_event.clear()
+        real_url = self._get_real_value(self.connection_url)
+        real_headers = self._get_real_value(self.connection_headers)
         try:
-            self.eio.connect(url, headers=headers, transports=transports,
+            self.eio.connect(real_url, headers=real_headers,
+                             transports=transports,
                              engineio_path=socketio_path)
         except engineio.exceptions.ConnectionError as exc:
             self._trigger_event(
                 'connect_error', '/',
                 exc.args[1] if len(exc.args) > 1 else exc.args[0])
             raise exceptions.ConnectionError(exc.args[0]) from None
+
+        if wait:
+            while self._connect_event.wait(timeout=wait_timeout):
+                self._connect_event.clear()
+                if set(self.namespaces) == set(self.connection_namespaces):
+                    break
+            if set(self.namespaces) != set(self.connection_namespaces):
+                self.disconnect()
+                raise exceptions.ConnectionError(
+                    'One or more namespaces failed to connect')
+
         self.connected = True
 
     def wait(self):
@@ -408,7 +449,6 @@ class Client(object):
         # later in _handle_eio_disconnect we invoke the disconnect handler
         for n in self.namespaces:
             self._send_packet(packet.Packet(packet.DISCONNECT, namespace=n))
-        self.connected = False
         self.eio.disconnect(abort=True)
 
     def get_sid(self, namespace=None):
@@ -460,6 +500,13 @@ class Client(object):
         """
         return self.eio.sleep(seconds)
 
+    def _get_real_value(self, value):
+        """Return the actual value, for parameters that can also be given as
+        callables."""
+        if not callable(value):
+            return value
+        return value()
+
     def _send_packet(self, pkt):
         """Send a Socket.IO packet to the server."""
         encoded_packet = pkt.encode()
@@ -484,6 +531,7 @@ class Client(object):
             self.logger.info('Namespace {} is connected'.format(namespace))
             self.namespaces[namespace] = (data or {}).get('sid', self.sid)
             self._trigger_event('connect', namespace=namespace)
+            self._connect_event.set()
 
     def _handle_disconnect(self, namespace):
         if not self.connected:
@@ -535,6 +583,7 @@ class Client(object):
         elif not isinstance(data, (tuple, list)):
             data = (data,)
         self._trigger_event('connect_error', namespace, *data)
+        self._connect_event.set()
         if namespace in self.namespaces:
             del self.namespaces[namespace]
         if namespace == '/':
@@ -553,6 +602,8 @@ class Client(object):
                 event, *args)
 
     def _handle_reconnect(self):
+        if self._reconnect_abort is None:  # pragma: no cover
+            self._reconnect_abort = self.eio.create_event()
         self._reconnect_abort.clear()
         reconnecting_clients.append(self)
         attempt_count = 0
@@ -573,6 +624,7 @@ class Client(object):
             try:
                 self.connect(self.connection_url,
                              headers=self.connection_headers,
+                             auth=self.connection_auth,
                              transports=self.connection_transports,
                              namespaces=self.connection_namespaces,
                              socketio_path=self.socketio_path)
@@ -593,8 +645,10 @@ class Client(object):
         """Handle the Engine.IO connection event."""
         self.logger.info('Engine.IO connection established')
         self.sid = self.eio.sid
+        real_auth = self._get_real_value(self.connection_auth)
         for n in self.connection_namespaces:
-            self._send_packet(packet.Packet(packet.CONNECT, namespace=n))
+            self._send_packet(packet.Packet(
+                packet.CONNECT, data=real_auth, namespace=n))
 
     def _handle_eio_message(self, data):
         """Dispatch Engine.IO messages."""
