@@ -255,10 +255,11 @@ class Server(object):
                      multiple arguments, use a tuple where each element is of
                      one of the types indicated above.
         :param to: The recipient of the message. This can be set to the
-                   session ID of a client to address only that client, or to
-                   to any custom room created by the application to address all
-                   the clients in that room, If this argument is omitted the
-                   event is broadcasted to all connected clients.
+                   session ID of a client to address only that client, to any
+                   any custom room created by the application to address all
+                   the clients in that room, or to a list of custom room
+                   names. If this argument is omitted the event is broadcasted
+                   to all connected clients.
         :param room: Alias for the ``to`` parameter.
         :param skip_sid: The session ID of a client to skip when broadcasting
                          to a room or to all clients. This can be used to
@@ -305,10 +306,11 @@ class Server(object):
                      multiple arguments, use a tuple where each element is of
                      one of the types indicated above.
         :param to: The recipient of the message. This can be set to the
-                   session ID of a client to address only that client, or to
-                   to any custom room created by the application to address all
-                   the clients in that room, If this argument is omitted the
-                   event is broadcasted to all connected clients.
+                   session ID of a client to address only that client, to any
+                   any custom room created by the application to address all
+                   the clients in that room, or to a list of custom room
+                   names. If this argument is omitted the event is broadcasted
+                   to all connected clients.
         :param room: Alias for the ``to`` parameter.
         :param skip_sid: The session ID of a client to skip when broadcasting
                          to a room or to all clients. This can be used to
@@ -619,7 +621,7 @@ class Server(object):
         else:
             self.eio.send(eio_sid, encoded_packet)
 
-    def _handle_connect(self, eio_sid, namespace):
+    def _handle_connect(self, eio_sid, namespace, data):
         """Handle a client connection request."""
         namespace = namespace or '/'
         sid = self.manager.connect(eio_sid, namespace)
@@ -628,8 +630,16 @@ class Server(object):
                 packet.CONNECT, {'sid': sid}, namespace=namespace))
         fail_reason = exceptions.ConnectionRefusedError().error_args
         try:
-            success = self._trigger_event('connect', namespace, sid,
-                                          self.environ[eio_sid])
+            if data:
+                success = self._trigger_event(
+                    'connect', namespace, sid, self.environ[eio_sid], data)
+            else:
+                try:
+                    success = self._trigger_event(
+                        'connect', namespace, sid, self.environ[eio_sid])
+                except TypeError:
+                    success = self._trigger_event(
+                        'connect', namespace, sid, self.environ[eio_sid], None)
         except exceptions.ConnectionRefusedError as exc:
             fail_reason = exc.error_args
             success = False
@@ -729,7 +739,7 @@ class Server(object):
         else:
             pkt = packet.Packet(encoded_packet=data)
             if pkt.packet_type == packet.CONNECT:
-                self._handle_connect(eio_sid, pkt.namespace)
+                self._handle_connect(eio_sid, pkt.namespace, pkt.data)
             elif pkt.packet_type == packet.DISCONNECT:
                 self._handle_disconnect(eio_sid, pkt.namespace)
             elif pkt.packet_type == packet.EVENT:
